"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PipelineEditorFactory = exports.getRuntimeIcon = exports.commandIDs = void 0;
const jsx_runtime_1 = require("react/jsx-runtime");
/*
 * Copyright 2018-2021 Elyra Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
const pipeline_editor_1 = require("@elyra/pipeline-editor");
const pipeline_services_1 = require("@elyra/pipeline-services");
const services_1 = require("@elyra/services");
const ui_components_1 = require("@elyra/ui-components");
const apputils_1 = require("@jupyterlab/apputils");
const coreutils_1 = require("@jupyterlab/coreutils");
const docregistry_1 = require("@jupyterlab/docregistry");
require("carbon-components/css/carbon-components.min.css");
const ui_components_2 = require("@jupyterlab/ui-components");
const algorithm_1 = require("@lumino/algorithm");
const signaling_1 = require("@lumino/signaling");
const core_1 = require("@material-ui/core");
const Alert_1 = __importDefault(require("@material-ui/lab/Alert"));
const react_1 = __importStar(require("react"));
const formDialogWidget_1 = require("./formDialogWidget");
const PipelineExportDialog_1 = require("./PipelineExportDialog");
const PipelineService_1 = require("./PipelineService");
const PipelineSubmissionDialog_1 = require("./PipelineSubmissionDialog");
const theme_1 = require("./theme");
const utils_1 = __importDefault(require("./utils"));
const PIPELINE_CLASS = 'elyra-PipelineEditor';
exports.commandIDs = {
    openPipelineEditor: 'pipeline-editor:open',
    openMetadata: 'elyra-metadata:open',
    openDocManager: 'docmanager:open',
    newDocManager: 'docmanager:new-untitled',
    saveDocManager: 'docmanager:save',
    submitScript: 'script-editor:submit',
    submitNotebook: 'notebook:submit',
    addFileToPipeline: 'pipeline-editor:add-node'
};
const runtimeIcons = [ui_components_1.kubeflowIcon, ui_components_1.airflowIcon];
const getRuntimeIcon = (runtime) => {
    for (const runtimeIcon of runtimeIcons) {
        if (`elyra:${runtime}` === runtimeIcon.name) {
            return runtimeIcon;
        }
    }
    return ui_components_1.pipelineIcon;
};
exports.getRuntimeIcon = getRuntimeIcon;
class PipelineEditorWidget extends apputils_1.ReactWidget {
    constructor(options) {
        super(options);
        this.browserFactory = options.browserFactory;
        this.shell = options.shell;
        this.commands = options.commands;
        this.addFileToPipelineSignal = options.addFileToPipelineSignal;
        this.context = options.context;
    }
    render() {
        var _a;
        return (jsx_runtime_1.jsx(PipelineWrapper, { context: this.context, browserFactory: this.browserFactory, shell: this.shell, commands: this.commands, addFileToPipelineSignal: this.addFileToPipelineSignal, widgetId: (_a = this.parent) === null || _a === void 0 ? void 0 : _a.id }, void 0));
    }
}
const NodeIcons = new Map([
    [
        'notebooks',
        'data:image/svg+xml;utf8,' + encodeURIComponent(ui_components_2.notebookIcon.svgstr)
    ],
    [
        'python-script',
        'data:image/svg+xml;utf8,' + encodeURIComponent(ui_components_1.pyIcon.svgstr)
    ],
    ['r-script', 'data:image/svg+xml;utf8,' + encodeURIComponent(ui_components_1.rIcon.svgstr)]
]);
const PipelineWrapper = ({ context, browserFactory, shell, commands, addFileToPipelineSignal, widgetId }) => {
    var _a, _b, _c, _d, _e;
    const ref = react_1.useRef(null);
    const [loading, setLoading] = react_1.useState(true);
    const [pipeline, setPipeline] = react_1.useState(null);
    const [panelOpen, setPanelOpen] = react_1.default.useState(false);
    const [alert, setAlert] = react_1.default.useState('');
    const [updatedNodes, setUpdatedNodes] = react_1.default.useState([]);
    const pipelineRuntime = (_d = (_c = (_b = (_a = pipeline === null || pipeline === void 0 ? void 0 : pipeline.pipelines) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.app_data) === null || _c === void 0 ? void 0 : _c.ui_data) === null || _d === void 0 ? void 0 : _d.runtime;
    const contextRef = react_1.useRef(context);
    react_1.useEffect(() => {
        const currentContext = contextRef.current;
        const changeHandler = () => {
            var _a, _b, _c, _d, _e, _f, _g, _h;
            const pipelineJson = currentContext.model.toJSON();
            if (((_b = (_a = pipelineJson === null || pipelineJson === void 0 ? void 0 : pipelineJson.pipelines) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.nodes) &&
                ((_d = (_c = pipelineJson === null || pipelineJson === void 0 ? void 0 : pipelineJson.pipelines) === null || _c === void 0 ? void 0 : _c[0]) === null || _d === void 0 ? void 0 : _d.nodes.length) > 0) {
                // Update to display actual value of runtime image
                for (const node of (_f = (_e = pipelineJson === null || pipelineJson === void 0 ? void 0 : pipelineJson.pipelines) === null || _e === void 0 ? void 0 : _e[0]) === null || _f === void 0 ? void 0 : _f.nodes) {
                    const app_data = node === null || node === void 0 ? void 0 : node.app_data;
                    if (app_data === null || app_data === void 0 ? void 0 : app_data.runtime_image) {
                        if ((_g = runtimeImages.current) === null || _g === void 0 ? void 0 : _g[app_data.runtime_image]) {
                            app_data.runtime_image = (_h = runtimeImages.current) === null || _h === void 0 ? void 0 : _h[app_data.runtime_image];
                        }
                    }
                }
            }
            setPipeline(pipelineJson);
            setLoading(false);
        };
        const loadNodes = (pipelineRuntime) => {
            PipelineService_1.PipelineService.getRuntimeImages().then((images) => {
                runtimeImages.current = images;
                PipelineService_1.PipelineService.getRuntimeComponents(pipelineRuntime !== null && pipelineRuntime !== void 0 ? pipelineRuntime : 'local').then((serverNodes) => __awaiter(void 0, void 0, void 0, function* () {
                    const newNodes = [];
                    for (const nodeCategory of serverNodes.categories) {
                        yield PipelineService_1.PipelineService.getComponentProperties(pipelineRuntime !== null && pipelineRuntime !== void 0 ? pipelineRuntime : 'local', nodeCategory.id).then((properties) => {
                            for (const node of nodeCategory.node_types) {
                                newNodes.push(node);
                                node.label = nodeCategory.label;
                                node.id = nodeCategory.id;
                                const nodeIcon = NodeIcons.get(nodeCategory.id);
                                if (!nodeIcon || nodeIcon === '') {
                                    node.image =
                                        'data:image/svg+xml;utf8,' +
                                            encodeURIComponent(exports.getRuntimeIcon(pipelineRuntime).svgstr);
                                }
                                else {
                                    node.image = nodeIcon;
                                }
                                node.description = nodeCategory.description;
                                node.properties = properties;
                                const index = node.properties.uihints.parameter_info.findIndex((p) => p.parameter_ref === 'runtime_image');
                                if (node.properties.uihints.parameter_info[index].data) {
                                    node.properties.uihints.parameter_info[index].data.items = Object.values(runtimeImages.current);
                                }
                                else {
                                    node.properties.uihints.parameter_info[index].data = {
                                        items: Object.values(runtimeImages.current)
                                    };
                                }
                            }
                        }, ui_components_1.RequestErrors.serverError);
                    }
                    setUpdatedNodes(newNodes);
                    changeHandler();
                }), ui_components_1.RequestErrors.serverError);
            }, ui_components_1.RequestErrors.serverError);
        };
        loadNodes();
        // Trigger a re-load of the nodes if the pipeline runtime changes
        const maybeLoadNodes = () => {
            var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k;
            const pipelineJSON = currentContext.model.toJSON();
            const pipelineRuntime = (_e = (_d = (_c = (_b = (_a = pipelineJSON === null || pipelineJSON === void 0 ? void 0 : pipelineJSON.pipelines) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.app_data) === null || _c === void 0 ? void 0 : _c.ui_data) === null || _d === void 0 ? void 0 : _d.runtime) === null || _e === void 0 ? void 0 : _e.name;
            if (pipelineRuntime !== ((_k = (_j = (_h = (_g = (_f = pipeline === null || pipeline === void 0 ? void 0 : pipeline.pipelines) === null || _f === void 0 ? void 0 : _f[0]) === null || _g === void 0 ? void 0 : _g.app_data) === null || _h === void 0 ? void 0 : _h.ui_data) === null || _j === void 0 ? void 0 : _j.runtime) === null || _k === void 0 ? void 0 : _k.name)) {
                loadNodes(pipelineRuntime);
            }
            else {
                changeHandler();
            }
        };
        currentContext.ready.then(maybeLoadNodes);
        currentContext.model.contentChanged.connect(maybeLoadNodes);
        return () => {
            currentContext.model.contentChanged.disconnect(changeHandler);
        };
    }, []);
    const onChange = react_1.useCallback((pipelineJson) => {
        var _a, _b, _c, _d, _e;
        if (contextRef.current.isReady) {
            if ((_b = (_a = pipelineJson === null || pipelineJson === void 0 ? void 0 : pipelineJson.pipelines) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.nodes) {
                // Update to store tag of runtime image
                for (const node of (_d = (_c = pipelineJson === null || pipelineJson === void 0 ? void 0 : pipelineJson.pipelines) === null || _c === void 0 ? void 0 : _c[0]) === null || _d === void 0 ? void 0 : _d.nodes) {
                    const app_data = node === null || node === void 0 ? void 0 : node.app_data;
                    if (app_data === null || app_data === void 0 ? void 0 : app_data.runtime_image) {
                        for (const tag in runtimeImages.current) {
                            if (((_e = runtimeImages.current) === null || _e === void 0 ? void 0 : _e[tag]) === (app_data === null || app_data === void 0 ? void 0 : app_data.runtime_image)) {
                                app_data.runtime_image = tag;
                            }
                        }
                    }
                }
            }
            contextRef.current.model.fromString(JSON.stringify(pipelineJson, null, 2));
        }
    }, []);
    const onError = (error) => {
        apputils_1.showDialog({
            title: 'Load pipeline failed!',
            body: jsx_runtime_1.jsxs("p", { children: [" ", error || '', " "] }, void 0),
            buttons: [apputils_1.Dialog.okButton()]
        }).then(() => {
            if (shell.currentWidget) {
                shell.currentWidget.close();
            }
        });
    };
    const runtimeImages = react_1.default.useRef({});
    const onFileRequested = (args) => {
        let currentExt = '';
        if (args && args.filters && args.filters.File) {
            currentExt = args.filters.File[0];
        }
        const filename = PipelineService_1.PipelineService.getWorkspaceRelativeNodePath(contextRef.current.path, '');
        return ui_components_1.showBrowseFileDialog(browserFactory.defaultBrowser.model.manager, {
            startPath: coreutils_1.PathExt.dirname(filename),
            multiselect: args.canSelectMany,
            filter: (model) => {
                const ext = coreutils_1.PathExt.extname(model.path);
                return currentExt === '' || currentExt === ext;
            }
        }).then((result) => {
            if (result.button.accept && result.value.length) {
                return result.value.map((val) => {
                    return val.path;
                });
            }
        });
    };
    const onPropertiesUpdateRequested = (args) => __awaiter(void 0, void 0, void 0, function* () {
        var _f;
        const path = PipelineService_1.PipelineService.getWorkspaceRelativeNodePath(contextRef.current.path, args.filename);
        const new_env_vars = yield services_1.ContentParser.getEnvVars(path).then((response) => response.map((str) => (str = str + '=')));
        const env_vars = (_f = args.env_vars) !== null && _f !== void 0 ? _f : [];
        const merged_env_vars = [
            ...env_vars,
            ...new_env_vars.filter((new_var) => !env_vars.some((old_var) => old_var.startsWith(new_var)))
        ];
        return { env_vars: merged_env_vars.filter(Boolean) };
    });
    const handleOpenFile = (data) => {
        for (let i = 0; i < data.selectedObjectIds.length; i++) {
            const node = pipeline.pipelines[0].nodes.find((node) => node.id === data.selectedObjectIds[i]);
            if (!node || !node.app_data || !node.app_data.filename) {
                continue;
            }
            const path = PipelineService_1.PipelineService.getWorkspaceRelativeNodePath(contextRef.current.path, node.app_data.filename);
            commands.execute(exports.commandIDs.openDocManager, { path });
        }
    };
    const cleanNullProperties = react_1.default.useCallback(() => {
        // Delete optional fields that have null value
        for (const node of pipeline === null || pipeline === void 0 ? void 0 : pipeline.pipelines[0].nodes) {
            if (node.app_data.cpu === null) {
                delete node.app_data.cpu;
            }
            if (node.app_data.memory === null) {
                delete node.app_data.memory;
            }
            if (node.app_data.gpu === null) {
                delete node.app_data.gpu;
            }
        }
    }, [pipeline === null || pipeline === void 0 ? void 0 : pipeline.pipelines]);
    const handleExportPipeline = react_1.useCallback(() => __awaiter(void 0, void 0, void 0, function* () {
        const pipelineJson = context.model.toJSON();
        // prepare pipeline submission details
        // Warn user if the pipeline has invalid nodes
        if (!pipelineJson) {
            setAlert('Failed export: Cannot export empty pipelines.');
            return;
        }
        const errorMessages = pipeline_services_1.validate(JSON.stringify(pipelineJson), updatedNodes);
        if (errorMessages && errorMessages.length > 0) {
            let errorMessage = '';
            for (const error of errorMessages) {
                errorMessage += error.message;
            }
            setAlert(`Failed export: ${errorMessage}`);
            return;
        }
        if (contextRef.current.model.dirty) {
            const dialogResult = yield apputils_1.showDialog({
                title: 'This pipeline contains unsaved changes. To submit the pipeline the changes need to be saved.',
                buttons: [
                    apputils_1.Dialog.cancelButton(),
                    apputils_1.Dialog.okButton({ label: 'Save and Submit' })
                ]
            });
            if (dialogResult.button && dialogResult.button.accept === true) {
                yield contextRef.current.save();
            }
            else {
                // Don't proceed if cancel button pressed
                return;
            }
        }
        const action = 'export pipeline';
        const runtimes = yield PipelineService_1.PipelineService.getRuntimes(true, action).catch(error => ui_components_1.RequestErrors.serverError(error));
        if (utils_1.default.isDialogResult(runtimes)) {
            // Open the runtimes widget
            runtimes.button.label.includes(PipelineService_1.RUNTIMES_NAMESPACE) &&
                shell.activateById(`elyra-metadata:${PipelineService_1.RUNTIMES_NAMESPACE}`);
            return;
        }
        const schema = yield PipelineService_1.PipelineService.getRuntimesSchema().catch(error => ui_components_1.RequestErrors.serverError(error));
        let title = 'Export pipeline';
        if (pipelineRuntime) {
            title = `Export pipeline for ${pipelineRuntime.display_name}`;
            const filteredRuntimeOptions = PipelineService_1.PipelineService.filterRuntimes(runtimes, pipelineRuntime.name);
            if (filteredRuntimeOptions.length === 0) {
                const runtimes = yield ui_components_1.RequestErrors.noMetadataError('runtime', 'export pipeline.', pipelineRuntime.display_name);
                if (utils_1.default.isDialogResult(runtimes)) {
                    if (runtimes.button.label.includes(PipelineService_1.RUNTIMES_NAMESPACE)) {
                        // Open the runtimes widget
                        shell.activateById(`elyra-metadata:${PipelineService_1.RUNTIMES_NAMESPACE}`);
                    }
                    return;
                }
                return;
            }
        }
        const dialogOptions = {
            title,
            body: formDialogWidget_1.formDialogWidget(jsx_runtime_1.jsx(PipelineExportDialog_1.PipelineExportDialog, { runtimes: runtimes, runtime: pipelineRuntime === null || pipelineRuntime === void 0 ? void 0 : pipelineRuntime.name, schema: schema }, void 0)),
            buttons: [apputils_1.Dialog.cancelButton(), apputils_1.Dialog.okButton()],
            defaultButton: 1,
            focusNodeSelector: '#runtime_config'
        };
        const dialogResult = yield ui_components_1.showFormDialog(dialogOptions);
        if (dialogResult.value == null) {
            // When Cancel is clicked on the dialog, just return
            return;
        }
        // prepare pipeline submission details
        const pipeline_path = contextRef.current.path;
        const pipeline_dir = coreutils_1.PathExt.dirname(pipeline_path);
        const pipeline_name = coreutils_1.PathExt.basename(pipeline_path, coreutils_1.PathExt.extname(pipeline_path));
        const pipeline_export_format = dialogResult.value.pipeline_filetype;
        let pipeline_export_path = pipeline_name + '.' + pipeline_export_format;
        // only prefix the '/' when pipeline_dir is non-empty
        if (pipeline_dir) {
            pipeline_export_path = pipeline_dir + '/' + pipeline_export_path;
        }
        const overwrite = dialogResult.value.overwrite;
        const runtime_config = dialogResult.value.runtime_config;
        const runtime = PipelineService_1.PipelineService.getRuntimeName(runtime_config, runtimes);
        PipelineService_1.PipelineService.setNodePathsRelativeToWorkspace(pipelineJson.pipelines[0], contextRef.current.path);
        cleanNullProperties();
        pipelineJson.pipelines[0].app_data.name = pipeline_name;
        pipelineJson.pipelines[0].app_data.runtime = runtime;
        pipelineJson.pipelines[0].app_data['runtime-config'] = runtime_config;
        pipelineJson.pipelines[0].app_data.source = coreutils_1.PathExt.basename(contextRef.current.path);
        PipelineService_1.PipelineService.exportPipeline(pipelineJson, pipeline_export_format, pipeline_export_path, overwrite).catch(error => ui_components_1.RequestErrors.serverError(error));
        PipelineService_1.PipelineService.setNodePathsRelativeToPipeline(pipelineJson.pipelines[0], contextRef.current.path);
    }), [context.model, pipeline === null || pipeline === void 0 ? void 0 : pipeline.pipelines, cleanNullProperties, shell]);
    const handleRunPipeline = react_1.useCallback(() => __awaiter(void 0, void 0, void 0, function* () {
        const pipelineJson = context.model.toJSON();
        // Check that all nodes are valid
        const errorMessages = pipeline_services_1.validate(JSON.stringify(pipelineJson), updatedNodes);
        if (errorMessages && errorMessages.length > 0) {
            let errorMessage = '';
            for (const error of errorMessages) {
                errorMessage += error.message;
            }
            setAlert(`Failed run: ${errorMessage}`);
            return;
        }
        if (contextRef.current.model.dirty) {
            const dialogResult = yield apputils_1.showDialog({
                title: 'This pipeline contains unsaved changes. To submit the pipeline the changes need to be saved.',
                buttons: [
                    apputils_1.Dialog.cancelButton(),
                    apputils_1.Dialog.okButton({ label: 'Save and Submit' })
                ]
            });
            if (dialogResult.button && dialogResult.button.accept === true) {
                yield contextRef.current.save();
            }
            else {
                // Don't proceed if cancel button pressed
                return;
            }
        }
        const pipelineName = coreutils_1.PathExt.basename(contextRef.current.path, coreutils_1.PathExt.extname(contextRef.current.path));
        const action = 'run pipeline';
        const runtimes = yield PipelineService_1.PipelineService.getRuntimes(false, action).catch(error => ui_components_1.RequestErrors.serverError(error));
        const schema = yield PipelineService_1.PipelineService.getRuntimesSchema().catch(error => ui_components_1.RequestErrors.serverError(error));
        const localRuntime = {
            name: 'local',
            display_name: 'Run in-place locally',
            schema_name: 'local'
        };
        runtimes.unshift(JSON.parse(JSON.stringify(localRuntime)));
        const localSchema = {
            name: 'local',
            display_name: 'Local Runtime'
        };
        schema.unshift(JSON.parse(JSON.stringify(localSchema)));
        let title = 'Run pipeline';
        if (pipelineRuntime) {
            title = `Run pipeline on ${pipelineRuntime.display_name}`;
            const filteredRuntimeOptions = PipelineService_1.PipelineService.filterRuntimes(runtimes, pipelineRuntime.name);
            if (filteredRuntimeOptions.length === 0) {
                const runtimes = yield ui_components_1.RequestErrors.noMetadataError('runtime', 'run pipeline.', pipelineRuntime.display_name);
                if (utils_1.default.isDialogResult(runtimes)) {
                    if (runtimes.button.label.includes(PipelineService_1.RUNTIMES_NAMESPACE)) {
                        // Open the runtimes widget
                        shell.activateById(`elyra-metadata:${PipelineService_1.RUNTIMES_NAMESPACE}`);
                    }
                    return;
                }
                return;
            }
        }
        const dialogOptions = {
            title,
            body: formDialogWidget_1.formDialogWidget(jsx_runtime_1.jsx(PipelineSubmissionDialog_1.PipelineSubmissionDialog, { name: pipelineName, runtimes: runtimes, runtime: pipelineRuntime === null || pipelineRuntime === void 0 ? void 0 : pipelineRuntime.name, schema: schema }, void 0)),
            buttons: [apputils_1.Dialog.cancelButton(), apputils_1.Dialog.okButton()],
            defaultButton: 1,
            focusNodeSelector: '#pipeline_name'
        };
        const dialogResult = yield ui_components_1.showFormDialog(dialogOptions);
        if (dialogResult.value === null) {
            // When Cancel is clicked on the dialog, just return
            return;
        }
        const runtime_config = dialogResult.value.runtime_config;
        const runtime = PipelineService_1.PipelineService.getRuntimeName(runtime_config, runtimes) || 'local';
        PipelineService_1.PipelineService.setNodePathsRelativeToWorkspace(pipelineJson.pipelines[0], contextRef.current.path);
        cleanNullProperties();
        pipelineJson.pipelines[0]['app_data']['name'] =
            dialogResult.value.pipeline_name;
        pipelineJson.pipelines[0]['app_data']['runtime'] = runtime;
        pipelineJson.pipelines[0]['app_data']['runtime-config'] = runtime_config;
        pipelineJson.pipelines[0]['app_data']['source'] = coreutils_1.PathExt.basename(contextRef.current.path);
        PipelineService_1.PipelineService.submitPipeline(pipelineJson, PipelineService_1.PipelineService.getDisplayName(dialogResult.value.runtime_config, runtimes)).catch(error => ui_components_1.RequestErrors.serverError(error));
        PipelineService_1.PipelineService.setNodePathsRelativeToPipeline(pipelineJson.pipelines[0], contextRef.current.path);
    }), [context.model, pipeline === null || pipeline === void 0 ? void 0 : pipeline.pipelines, cleanNullProperties, shell]);
    const handleClearPipeline = react_1.useCallback((data) => __awaiter(void 0, void 0, void 0, function* () {
        return apputils_1.showDialog({
            title: 'Clear Pipeline',
            body: 'Are you sure you want to clear the pipeline?',
            buttons: [apputils_1.Dialog.cancelButton(), apputils_1.Dialog.okButton({ label: 'Clear' })]
        }).then(result => {
            if (result.button.accept) {
                // select all canvas elements
                contextRef.current.model.fromString('');
            }
        });
    }), []);
    const onAction = react_1.useCallback((args) => {
        console.log(args.type);
        switch (args.type) {
            case 'save':
                contextRef.current.save();
                break;
            case 'run':
                handleRunPipeline();
                break;
            case 'clear':
                handleClearPipeline(args.payload);
                break;
            case 'export':
                handleExportPipeline();
                break;
            case 'toggleOpenPanel':
                setPanelOpen(!panelOpen);
                break;
            case 'properties':
                setPanelOpen(true);
                break;
            case 'openRuntimes':
                shell.activateById(`elyra-metadata:${PipelineService_1.RUNTIMES_NAMESPACE}`);
                break;
            case 'openFile':
                commands.execute(exports.commandIDs.openDocManager, { path: args.payload });
                break;
            default:
                break;
        }
    }, [
        handleRunPipeline,
        handleClearPipeline,
        handleExportPipeline,
        panelOpen,
        shell,
        commands
    ]);
    const toolbar = {
        leftBar: [
            {
                action: 'run',
                label: 'Run Pipeline',
                enable: true
            },
            {
                action: 'save',
                label: 'Save Pipeline',
                enable: true,
                iconEnabled: ui_components_1.IconUtil.encode(ui_components_1.savePipelineIcon),
                iconDisabled: ui_components_1.IconUtil.encode(ui_components_1.savePipelineIcon)
            },
            {
                action: 'export',
                label: 'Export Pipeline',
                enable: true,
                iconEnabled: ui_components_1.IconUtil.encode(ui_components_1.exportPipelineIcon),
                iconDisabled: ui_components_1.IconUtil.encode(ui_components_1.exportPipelineIcon)
            },
            {
                action: 'clear',
                label: 'Clear Pipeline',
                enable: true,
                iconEnabled: ui_components_1.IconUtil.encode(ui_components_1.clearPipelineIcon),
                iconDisabled: ui_components_1.IconUtil.encode(ui_components_1.clearPipelineIcon)
            },
            {
                action: 'openRuntimes',
                label: 'Open Runtimes',
                enable: true,
                iconEnabled: ui_components_1.IconUtil.encode(ui_components_1.runtimesIcon),
                iconDisabled: ui_components_1.IconUtil.encode(ui_components_1.runtimesIcon)
            },
            { action: 'undo', label: 'Undo' },
            { action: 'redo', label: 'Redo' },
            { action: 'cut', label: 'Cut' },
            { action: 'copy', label: 'Copy' },
            { action: 'paste', label: 'Paste' },
            { action: 'createAutoComment', label: 'Add Comment', enable: true },
            { action: 'deleteSelectedObjects', label: 'Delete' },
            {
                action: 'arrangeHorizontally',
                label: 'Arrange Horizontally',
                enable: true
            },
            {
                action: 'arrangeVertically',
                label: 'Arrange Vertically',
                enable: true
            }
        ],
        rightBar: [
            {
                action: '',
                label: `Runtime: ${(_e = pipelineRuntime === null || pipelineRuntime === void 0 ? void 0 : pipelineRuntime.display_name) !== null && _e !== void 0 ? _e : 'Generic'}`,
                incLabelWithIcon: 'before',
                enable: false,
                kind: 'tertiary',
                // TODO: use getRuntimeIcon
                iconEnabled: ui_components_1.IconUtil.encode((pipelineRuntime === null || pipelineRuntime === void 0 ? void 0 : pipelineRuntime.name) === 'kfp'
                    ? ui_components_1.kubeflowIcon
                    : (pipelineRuntime === null || pipelineRuntime === void 0 ? void 0 : pipelineRuntime.name) === 'airflow'
                        ? ui_components_1.airflowIcon
                        : ui_components_1.pipelineIcon)
            },
            {
                action: 'toggleOpenPanel',
                label: panelOpen ? 'Close Panel' : 'Open Panel',
                enable: true,
                iconTypeOverride: panelOpen ? 'paletteOpen' : 'paletteClose'
            }
        ]
    };
    const [defaultPosition, setDefaultPosition] = react_1.useState(10);
    const handleAddFileToPipeline = react_1.useCallback((location) => {
        var _a;
        const fileBrowser = browserFactory.defaultBrowser;
        // Only add file to pipeline if it is currently in focus
        if (((_a = shell.currentWidget) === null || _a === void 0 ? void 0 : _a.id) !== widgetId) {
            return;
        }
        let failedAdd = 0;
        let position = 0;
        const missingXY = !location;
        // if either x or y is undefined use the default coordinates
        if (missingXY) {
            position = defaultPosition;
            location = {
                x: 75,
                y: 85
            };
        }
        algorithm_1.toArray(fileBrowser.selectedItems()).map((item) => {
            var _a, _b, _c;
            if (PipelineService_1.PipelineService.isSupportedNode(item)) {
                // read the file contents
                // create a notebook widget to get a string with the node content then dispose of it
                // let itemContent: string;
                if (item.type == 'notebook') {
                    const fileWidget = fileBrowser.model.manager.open(item.path);
                    // itemContent = (fileWidget as NotebookPanel).content.model.toString();
                    fileWidget === null || fileWidget === void 0 ? void 0 : fileWidget.dispose();
                }
                item.op = PipelineService_1.PipelineService.getNodeType(item.path);
                item.path = PipelineService_1.PipelineService.getPipelineRelativeNodePath(contextRef.current.path, item.path);
                item.x = ((_a = location === null || location === void 0 ? void 0 : location.x) !== null && _a !== void 0 ? _a : 0) + position;
                item.y = ((_b = location === null || location === void 0 ? void 0 : location.y) !== null && _b !== void 0 ? _b : 0) + position;
                const success = (_c = ref.current) === null || _c === void 0 ? void 0 : _c.addFile(item);
                if (success) {
                    position += 20;
                }
                else {
                    // handle error
                }
            }
            else {
                failedAdd++;
            }
        });
        // update position if the default coordinates were used
        if (missingXY) {
            setDefaultPosition(position);
        }
        if (failedAdd) {
            return apputils_1.showDialog({
                title: 'Unsupported File(s)',
                body: 'Currently, only selected notebook and python script files can be added to a pipeline',
                buttons: [apputils_1.Dialog.okButton()]
            });
        }
        return;
    }, [browserFactory.defaultBrowser, defaultPosition, shell, widgetId]);
    const handleDrop = (e) => __awaiter(void 0, void 0, void 0, function* () {
        handleAddFileToPipeline({ x: e.offsetX, y: e.offsetY });
    });
    react_1.useEffect(() => {
        const handleSignal = () => {
            handleAddFileToPipeline();
        };
        addFileToPipelineSignal.connect(handleSignal);
        return () => {
            addFileToPipelineSignal.disconnect(handleSignal);
        };
    }, [addFileToPipelineSignal, handleAddFileToPipeline]);
    const handleClose = (event, reason) => {
        if (reason === 'clickaway') {
            return;
        }
        setAlert('');
    };
    if (loading) {
        return jsx_runtime_1.jsx("div", { children: "loading" }, void 0);
    }
    return (jsx_runtime_1.jsxs(pipeline_editor_1.ThemeProvider, Object.assign({ theme: theme_1.theme }, { children: [jsx_runtime_1.jsx(core_1.Snackbar, Object.assign({ open: alert !== '', autoHideDuration: 6000, onClose: handleClose }, { children: jsx_runtime_1.jsx(Alert_1.default, Object.assign({ severity: 'error', onClose: handleClose }, { children: alert }), void 0) }), void 0),
            jsx_runtime_1.jsx(ui_components_1.Dropzone, Object.assign({ onDrop: handleDrop }, { children: jsx_runtime_1.jsx(pipeline_editor_1.PipelineEditor, { ref: ref, nodes: updatedNodes, toolbar: toolbar, pipeline: pipeline, onAction: onAction, onChange: onChange, onDoubleClickNode: handleOpenFile, onError: onError, onFileRequested: onFileRequested, onPropertiesUpdateRequested: onPropertiesUpdateRequested }, void 0) }), void 0)] }), void 0));
};
class PipelineEditorFactory extends docregistry_1.ABCWidgetFactory {
    constructor(options) {
        super(options);
        this.browserFactory = options.browserFactory;
        this.shell = options.shell;
        this.commands = options.commands;
        this.addFileToPipelineSignal = new signaling_1.Signal(this);
    }
    createNewWidget(context) {
        // Creates a blank widget with a DocumentWidget wrapper
        const props = {
            shell: this.shell,
            commands: this.commands,
            browserFactory: this.browserFactory,
            context: context,
            addFileToPipelineSignal: this.addFileToPipelineSignal
        };
        const content = new PipelineEditorWidget(props);
        const widget = new docregistry_1.DocumentWidget({ content, context });
        widget.addClass(PIPELINE_CLASS);
        widget.title.icon = ui_components_1.pipelineIcon;
        return widget;
    }
}
exports.PipelineEditorFactory = PipelineEditorFactory;
//# sourceMappingURL=PipelineEditorWidget.js.map